'use strict';

const AWS = require('aws-sdk');
const errors = require('../../common/errors');

const dynamodb = new AWS.DynamoDB.DocumentClient();

/**
 * @api {post} /vessels Create new journey
 * @apiGroup Journeys
 * 
 * @apiSuccess {String}   vessel_name             Vessel name.
 * @apiSuccess {Number}   estimated_arrival_time  Estimated arrival time.
 * @apiSuccess {Object}   last_port               Last port information.
 * @apiSuccess {String}   last_port.city          Last port name.
 * @apiSuccess {String}   last_port.country       Last port country.
 * @apiSuccess {Object}   next_port               Next port information.
 * @apiSuccess {String}   next_port.city          Next port name.
 * @apiSuccess {String}   next_port.country       Next port country.
 * @apiSuccess {Number}   status                  Journey status.
 * @apiSuccess {Number}   created_at              Timestamp of creation.
 * @apiSuccess {Number}   updated_at              Timestamp of last update.
 * 
 * @apiError             ValidationError      Couldn't create the journey
 * @apiError (Error 5xx) InternalServerError  Couldn't create the journey
 */
module.exports.create = (event, context, callback) => {
  const timestamp = new Date().getTime();
  const data = JSON.parse(event.body);

  /* Validate input data */
  const error_msgs = [];

  if (typeof data.vessel_name !== 'string') {
    error_msgs.push('Vessel name is not present.');
  }

  if (typeof data.estimated_arrival_time !== 'string') {
    error_msgs.push('Estimated arrival time is not present.');
  }

  if (typeof data.last_port !== 'object') {
    error_msgs.push('Last port is not present.');
  } else {
    if (typeof data.last_port.city !== 'string') {
      error_msgs.push('Last port city is not present.');
    }

    if (typeof data.last_port.country !== 'string') {
      error_msgs.push('Last country is not present.');
    }
  }

  if (typeof data.next_port !== 'object') {
    error_msgs.push('Next port is not present.');
  } else {
    if (typeof data.next_port.city !== 'string') {
      error_msgs.push('Next port city is not present.');
    }

    if (typeof data.next_port.country !== 'string') {
      error_msgs.push('Next country is not present.');
    }
  }

  if (error_msgs.length > 0) {
    error_msgs.unshift("Couldn't create the journey:");

    callback(null, errors.validationError(error_msgs.join(' ')));
    return;
  }

  const params = {
    TableName: process.env.JOURNEYS_TABLE,
    Item: {
        vessel_name: data.vessel_name,
        estimated_arrival_time: 
          new Date(data.estimated_arrival_time).getTime(),
        status: 0,
        last_port: {
            city: data.last_port.city,
            country: data.last_port.country
        },
        next_port: {
            city: data.last_port.city,
            country: data.last_port.country
        },
        created_at: timestamp,
        updated_at: timestamp
    }
  };
    
  dynamodb.put(params, (error) => {
    if (error) {
      callback(null, errors.internalServerError("Couldn't create the journey"));
      return;
    }

    const response = {
      statusCode: 200,
      body: JSON.stringify(params.Item)
    };

    callback(null, response);
  });
};