'use strict';

const AWS = require('aws-sdk');
const errors = require('../../common/errors');

const dynamodb = new AWS.DynamoDB.DocumentClient();

/**
 * @api {get} /journeys/:vessel_name/:expected_time Request journey information
 * @apiGroup Journeys
 * 
 * @apiParam {String} vessel_name Vessel's name.
 * @apiParam {Number} expected_time Estimated arrival time.
 * 
 * @apiSuccess {Number}   estimated_arrival_time   Estimated arrival time.
 * @apiSuccess {Object}   vessel                   Vessel information.
 * @apiSuccess {String}   vessel.name              Vessel name.
 * @apiSuccess {Object}   vessel.home_port         Home port information.
 * @apiSuccess {String}   vessel.home_port.city    Home port name.
 * @apiSuccess {String}   vessel.home_port.country Home port country.
 * @apiSuccess {Object}   last_port                Last port information.
 * @apiSuccess {String}   last_port.city           Last port name.
 * @apiSuccess {String}   last_port.country        Last port country.
 * @apiSuccess {Object}   next_port                Next port information.
 * @apiSuccess {String}   next_port.city           Next port name.
 * @apiSuccess {String}   next_port.country        Next port country.
 * @apiSuccess {Number}   status                   Journey status.
 * @apiSuccess {Number}   actual_arrival_time      Actual arrival time (Optional)
 * @apiSuccess {Number}   actual_departure_time    Actual departure time (Optional)
 * @apiSuccess {Number}   created_at               Timestamp of creation.
 * @apiSuccess {Number}   updated_at               Timestamp of last update.
 * 
 * @apiError             NotFoundError1        Couldn't find the journey
 * @apiError             NotFoundError2        Journey's vessel doesn't exist
 * @apiError (Error 5xx) InternalServerError   Couldn't fetch the journey
 */
module.exports.show = (event, context, callback) => {
  const params = {
    TableName: process.env.JOURNEYS_TABLE,
    ProjectionExpression: "estimated_arrival_time, last_port, next_port, #journey_status, created_at, updated_at",
    Key: {
      vessel_name: event.pathParameters.vessel_name,
      estimated_arrival_time: parseInt(event.pathParameters.expected_time)
    },
    ExpressionAttributeNames: {
      "#journey_status": "status"
    }
  };

  dynamodb.get(params, (error, result) => {
    if (error) {
      callback(null, errors.internalServerError("Couldn't fetch the journey"));
      return;
    }

    if (typeof result.Item !== 'object') {
      callback(null, errors.notFoundError("Couldn't find the journey"));
      return;
    }

    var journey = result.Item;

    const vessel_params = {
      TableName: process.env.VESSELS_TABLE,
      ProjectionExpression: "home_port, #vessel_name",
      Key: {
        name: event.pathParameters.vessel_name
      },
      ExpressionAttributeNames: {
        "#vessel_name": "name"
      }
    };

    dynamodb.get(vessel_params, (v_error, v_result) => {
      if (v_error) {
        callback(null, errors.internalServerError("Couldn't fetch the journey"));
        return;
      }

      if (typeof v_result.Item !== 'object') {
        callback(null, errors.notFoundError("Journey's vessel doesn't exist"));
        return;
      }

      journey.vessel = v_result.Item;

      const response = {
        statusCode: 200,
        body: JSON.stringify(journey)
      };

      callback(null, response);
    });
  });
};
