'use strict';

const AWS = require('aws-sdk');
const errors = require('../../common/errors');

const dynamodb = new AWS.DynamoDB.DocumentClient();

/**
 * @api {put} /journeys/:vessel_name/:expected_time Update journey information
 * @apiGroup Journeys
 * 
 * @apiParam {String} vessel_name            Vessel's name.
 * @apiParam {Number} expected_time          Estimated arrival time.
 * @apiParam {Number} status                 New journey status.
 * @apiParam {Number} actual_arrival_time    Actual arrival time (Required when new status is 2)
 * @apiParam {Number} actual_departure_time  Actual departure time (Required when new status is 3)
 * 
 * @apiSuccessExample {none} Success-Response:
 *     HTTP/1.1 204 No Content
 * 
 * @apiError             NotFoundError         Couldn't fetch the journey
 * @apiError             ValidationError       Couldn't update the journey
 * @apiError (Error 5xx) InternalServerError1  Couldn't fetch the journey
 * @apiError (Error 5xx) InternalServerError2  Couldn't update the journey
 */
module.exports.update = (event, context, callback) => {
  const data = JSON.parse(event.body);
  const new_status = parseInt(data.status);
  const timestamp = new Date().getTime();
  
  if (new_status < 1 || new_status > 4) {
    callback(null, errors.validationError("Couldn't update the journey: New status must be in range from 1 to 4."));
    return;
  }
  
  var params = {
    TableName: process.env.JOURNEYS_TABLE,
    ProjectionExpression: "#journey_status",
    Key: {
      vessel_name: event.pathParameters.vessel_name,
      estimated_arrival_time: parseInt(event.pathParameters.expected_time)
    },
    ExpressionAttributeNames: {
      "#journey_status": "status"
    }
  };

  var status = null;
  var item = null;

  dynamodb.get(params, (error, result) => {
    if (error) {
      callback(null, errors.internalServerError("Couldn't fetch the journey"));
      return;
    }

    if (typeof result.Item !== 'object') {
      callback(null, errors.notFoundError("Couldn't fetch the journey"));
      return;
    }

    status = result.Item.status;

    var update_params = {
      TableName: process.env.JOURNEYS_TABLE,
      Key: {
        vessel_name: event.pathParameters.vessel_name,
        estimated_arrival_time: parseInt(event.pathParameters.expected_time)
      },
      ExpressionAttributeNames: {
        "#journey_status": "status"
      }
    };

    if (new_status == 1 || new_status == 4) {
      if (new_status == 1 && status != 0) {
        callback(null, errors.validationError("Couldn't update the journey: Changing to status 1 requires status 0."));
        return;
      }
  
      update_params.ExpressionAttributeValues = {
        ':status': new_status,
        ':updated_at': timestamp
      };
      update_params.UpdateExpression = 'SET #journey_status = :status, updated_at = :updated_at';
    }

    if (new_status == 2) {
      if (status != 1) {
        callback(null, errors.validationError("Couldn't update the journey: Changing to status 2 requires status 1."));
        return;
      }
  
      if (typeof data.actual_arrival_time !== 'string') {
        callback(null, errors.validationError("Couldn't update the journey: actual_arrival_time must be present"));
        return;
      }
  
      update_params.ExpressionAttributeValues = {
        ':status': new_status,
        ':actual_arrival_time': new Date(data.actual_arrival_time).getTime(),
        ':updated_at': timestamp
      };
      update_params.UpdateExpression = 'SET #journey_status = :status, actual_arrival_time = :actual_arrival_time, updated_at = :updated_at';
    }

    if (new_status == 3) {
      if (status != 2) {
        callback(null, errors.validationError("Couldn't update the journey: Changing to status 3 requires status 2."));
        return;
      }
  
      if (typeof data.actual_departure_time !== 'string') {
        callback(null, errors.validationError("Couldn't update the journey: actual_departure_time must be present"));
        return;
      }
  
      update_params.ExpressionAttributeValues = {
        ':status': new_status,
        ':actual_departure_time': new Date(data.actual_departure_time).getTime(),
        ':updated_at': timestamp
      };
      update_params.UpdateExpression = 'SET #journey_status = :status, actual_departure_time = :actual_departure_time, updated_at = :updated_at';
    }

    dynamodb.update(update_params, (u_error, u_result) => {
      if (u_error) {
        console.log(u_error);
        callback(null, errors.internalServerError("Couldn't update the journey"));
        return;
      }
  
      const response = {
        statusCode: 204
      };
  
      callback(null, response);
    });
  }); 
};
