'use strict';

const AWS = require('aws-sdk');
const errors = require('../../common/errors');

const dynamodb = new AWS.DynamoDB.DocumentClient();

/**
 * @api {post} /vessels Create new vessel
 * @apiGroup Vessels
 *
 * @apiParam {String} name               Name of the vessel.
 * @apiParam {Object} home_port          Home port information.
 * @apiParam {String} home_port.city     Home port name.
 * @apiParam {String} home_port.country  Home port country.
 * 
 * @apiSuccess {String} name               Name of the vessel.
 * @apiSuccess {Object} home_port          Home port information.
 * @apiSuccess {String} home_port.city     Home port name.
 * @apiSuccess {String} home_port.country  Home port country.
 * @apiSuccess {Number} created_at         Timestamp of creation.
 * @apiSuccess {Number} updated_at         Timestamp of last update.
 * 
 * @apiError             ValidationError      Couldn't create the vessel
 * @apiError (Error 5xx) InternalServerError  Couldn't create the vessel
 */
module.exports.create = (event, context, callback) => {
  const timestamp = new Date().getTime();
  const data = JSON.parse(event.body);

  const error_msgs = [];

  if (typeof data.name !== 'string') {
    error_msgs.push('Name is not present.');
  }

  if (typeof data.home_port !== 'object') {
    error_msgs.push('Home port is not present.');
  } else {
    if (typeof data.home_port.city !== 'string') {
      error_msgs.push('Home port city is not present.');
    }

    if (typeof data.home_port.country !== 'string') {
      error_msgs.push('Home country is not present.');
    }
  }

  if (error_msgs.length > 0) {
    error_msgs.unshift("Couldn't create vessel:");

    callback(null, errors.validationError(error_msgs.join(' ')));
    return;
  }

  /* Prepare insertion into database */
  const params = {
    TableName: process.env.VESSELS_TABLE,
    Item: {
        name: data.name,
        home_port: {
            city: data.home_port.city,
            country: data.home_port.country
        },
        created_at: timestamp,
        updated_at: timestamp
    }
  };
    
  dynamodb.put(params, (error) => {
    if (error) {
      callback(null, errors.internalServerError("Couldn't create the vessel"));
      return;
    }

    const response = {
      statusCode: 200,
      body: JSON.stringify(params.Item)
    };

    callback(null, response);
  });
};
