'use strict';

const AWS = require('aws-sdk');
const errors = require('../../common/errors');

const dynamodb = new AWS.DynamoDB.DocumentClient();

/**
 * @api {get} /vessels/:name Request vessel information
 * @apiGroup Vessels
 *
 * @apiParam {String} name Vessels name.
 *
 * @apiSuccess {String}   name                             Name of the vessel.
 * @apiSuccess {Object}   home_port                        Home port information.
 * @apiSuccess {String}   home_port.city                   Home port name.
 * @apiSuccess {String}   home_port.country                Home port country.
 * @apiSuccess {Object[]} journeys                         List of journeys
 * @apiSuccess {Number}   journeys.estimated_arrival_time  Estimated arrival time.
 * @apiSuccess {Object}   journeys.last_port               Last port information.
 * @apiSuccess {String}   journeys.last_port.city          Last port name.
 * @apiSuccess {String}   journeys.last_port.country       Last port country.
 * @apiSuccess {Object}   journeys.next_port               Next port information.
 * @apiSuccess {String}   journeys.next_port.city          Next port name.
 * @apiSuccess {String}   journeys.next_port.country       Next port country.
 * @apiSuccess {Number}   journeys.status                  Journey status.
 * @apiSuccess {Number}   created_at                       Timestamp of creation.
 * @apiSuccess {Number}   updated_at                       Timestamp of last update.
 *              
 * @apiError             NotFoundError        Couldn't find the vessel
 * @apiError (Error 5xx) InternalServerError  Couldn't fetch the vessel
 */
module.exports.show = (event, context, callback) => {
  const params = {
    TableName: process.env.VESSELS_TABLE,
    Key: {
      name: event.pathParameters.name
    }
  };

  dynamodb.get(params, (error, result) => {
    if (error) {
      callback(null, errors.internalServerError("Couldn't fetch the vessel"));
      return;
    }

    if (typeof result.Item !== 'object') {
      callback(null, errors.notFoundError("Couldn't find the vessel"));
      return;
    }

    var vessel = result.Item;

    const journeys_params = {
      TableName: process.env.JOURNEYS_TABLE,
      ProjectionExpression: "last_port, next_port, estimated_arrival_time, #journey_status",
      KeyConditionExpression: "vessel_name = :vessel_name",
      ExpressionAttributeValues: {
        ":vessel_name": event.pathParameters.name
      },
      ExpressionAttributeNames: {
        "#journey_status": "status"
      }
    };

    dynamodb.query(journeys_params, (j_error, j_result) => {
      if (j_error) {
        callback(null, errors.internalServerError("Couldn't fetch the vessel"));
        return;
      }

      vessel.journeys = j_result.Items;

      const response = {
        statusCode: 200,
        body: JSON.stringify(vessel)
      };

      callback(null, response);
    });
  });
};
