'use strict';

const AWS = require('aws-sdk');
const errors = require('../../common/errors');

const dynamodb = new AWS.DynamoDB.DocumentClient();

/**
 * @api {put} /vessels/:name Update vessel information
 * @apiGroup Vessels
 *
 * @apiParam {String} name               Vessel's name.
 * @apiParam {Object} home_port          Home port information.
 * @apiParam {String} home_port.city     Home port name.
 * @apiParam {String} home_port.country  Home port country.
 * 
 * @apiSuccessExample {none} Success-Response:
 *     HTTP/1.1 204 No Content
 * 
 * @apiError (Error 5xx) InternalServerError  Couldn't update the vessel
 */
module.exports.update = (event, context, callback) => {
  const timestamp = new Date().getTime();
  const data = JSON.parse(event.body);

  const params = {
    TableName: process.env.VESSELS_TABLE,
    Key: {
      name: event.pathParameters.name
    },
    ExpressionAttributeValues: {
      ':city': data.home_port.city,
      ':country': data.home_port.country,
      ':updated_at': timestamp,
    },
    UpdateExpression: 'SET home_port.city = :city, home_port.country = :country, updated_at = :updated_at'
  };

  dynamodb.update(params, (error, result) => {
    if (error) {
      callback(null, errors.internalServerError("Couldn't update the vessel"));
      return;
    }

    const response = {
      statusCode: 204
    };

    callback(null, response);
  });
};
